# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from hydra_agent import Rac
from hydra_agent import config
from hydra_agent.rac.cluster_manager.cluster import Cluster


class ClusterManager(Rac):
    def __init__(self, settings=None):
        if not settings:
            settings = config.rac
        super().__init__(settings)

    def list(self):
        """Returns list of `Cluster`"""

        output = super()._run_command(["cluster", "list"])
        for params in self._parse_output(output):
            yield Cluster.from_dict(params)



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import aiohttp.web

from hydra_agent.api.handler.base import BaseHandler
from hydra_agent.rac.cluster_manager.cluster import Cluster
from hydra_agent.rac.session_manager import SessionManager


class SessionHandler(BaseHandler):
    @property
    def routes(self):
        return [
            aiohttp.web.get(self.build_route("/clusters/{cluster_id}/sessions"), self.read),
            aiohttp.web.get(self.build_route("/clusters/{cluster_id}/sessions/{session_id}"), self.read),
        ]

    async def read(self, request):
        cluster_id = request.match_info.get("cluster_id")
        session_id = request.match_info.get("session_id", None)
        return aiohttp.web.json_response(get_sessions(cluster_id, session_id))


def get_sessions(cluster_id, session_id=None):
    if session_id:
        return get_info(cluster_id, session_id)
    else:
        return get_list(cluster_id)


def get_list(cluster_id):
    sessions = []
    for session in SessionManager(Cluster(cluster_id)).list():
        sessions.append(session.to_dict())
    return sessions


def get_info(cluster_id, session_id):
    for session in get_list(cluster_id):
        if session["id"] == session_id:
            return session



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class InfoBase:
    def __init__(self, raw="", id=""):
        if id:
            self.id = id.strip()
        else:
            result = self.__parse(raw)
            self.id = result["infobase"]
            self.name = result["name"]
            self.description = result["descr"]

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __parse(self, raw):
        result = {}
        for i in raw.strip().split("\n"):
            k, v = i.split(":", 1)
            result[k.strip()] = v.strip()
        return result



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os.path

from hydra_agent import config
from hydra_agent.utils.system import run_command
from hydra_agent.utils import is_windows


class Rac:
    def __init__(self, settings=None):
        if not settings:
            settings = config.rac
        self.path = os.path.join(settings.path, "rac" + (".exe" if is_windows() else ""))
        self.host = settings.host
        self.port = settings.port

    @property
    def version(self):
        return self._run_command([self.path, "--version"])

    def _run_command(self, args):
        args = self.__add_server([self.path]) + args
        return run_command(args)

    def __add_server(self, args):
        args.append(f"{self.host}:{self.port}")
        return args

    @staticmethod
    def _parse_output(output):
        for block in output.strip().split("\n\n"):
            if block:
                item = {}
                for line in block.strip().split("\n"):
                    k, v = line.split(":", 1)
                    item[k.strip()] = v.strip()
                yield item



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from hydra_agent import Rac
from hydra_agent.rac.infobase_manager.infobase import InfoBase


class InfoBaseManager(Rac):
    def __init__(self, config, cluster):
        super().__init__(config)
        self.cluster = cluster

    def list(self):
        """Returns list of `InfoBase`"""

        result = []
        for ib_info in self._run_command(["infobase", "summary", "list"]).split("\n\n"):
            if ib_info.strip():
                result.append(InfoBase(ib_info))
        return result

    def _run_command(self, args):
        args.append(f"--cluster={self.cluster.id}")
        return super()._run_command(args)



from .base_config import BaseConfig


class RacConfig(BaseConfig):
    def __init__(self, path, host, port):
        self.path = path
        self.host = host
        self.port = port

    def __str__(self):
        return f"path={self.path}, host={self.host}, port={self.port}"



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from hydra_agent.config.base_config import BaseConfig


class ApiConfig(BaseConfig):
    def __init__(self, host, port, debug):
        self.host = host
        self.port = port
        self.debug = debug

    def __str__(self):
        return f"host={self.host}, port={self.port}, debug={self.debug}"



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import io
import os

from setuptools import find_packages, setup

NAME = "hydra_agent"
here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, NAME, "__version__.py")) as f:
    exec(f.read(), about)

with io.open(os.path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

setup(
    name=NAME,
    version=about[NAME],
    description="Cross-platform remote administration agent for 1C:Enterprise",
    long_description=long_description,
    author="Vladimir Bondarevskiy",
    author_email="vbondarevsky@gmail.com",
    url="https://github.com/vbondarevsky/hydra_agent",
    license="https://www.gnu.org/licenses/gpl-3.0",
    packages=find_packages(exclude=["tests", "tests.*"]),
    python_requires=">=3",
    install_requires=[
        "pyyaml",
        "aiohttp"
    ],
    zip_safe=True,
    entry_points={
        "console_scripts": [
            "hydra_agent = hydra_agent.main:run"
        ]
    },
    classifiers=[
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from hydra_agent.rac.cluster_manager.load_balancing_mode import LoadBalancingMode
from hydra_agent.rac.cluster_manager.security_level import SecurityLevel


class Cluster:
    id = ""
    host = ""
    port = 0
    name = ""
    expiration_timeout = 0
    lifetime_limit = 0
    max_memory_size = 0
    max_memory_time_limit = 0
    security_level = SecurityLevel.InsecureConnection
    session_fault_tolerance_level = 0
    load_balancing_mode = LoadBalancingMode.Performance
    errors_count_threshold = 0
    kill_problem_processes = False

    def __init__(self, cluster_id):
        self.id = cluster_id.strip()

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    @staticmethod
    def from_dict(params):
        cluster = Cluster(params["cluster"])
        cluster.host = params["host"]
        cluster.port = int(params["port"])
        cluster.name = params["name"]
        cluster.expiration_timeout = int(params["expiration-timeout"])
        cluster.lifetime_limit = int(params["lifetime-limit"])
        cluster.max_memory_size = int(params["max-memory-size"])
        cluster.max_memory_time_limit = int(params["max-memory-time-limit"])
        cluster.security_level = SecurityLevel(int(params["security-level"]))
        cluster.session_fault_tolerance_level = int(params["session-fault-tolerance-level"])
        cluster.load_balancing_mode = LoadBalancingMode(params["load-balancing-mode"])
        cluster.errors_count_threshold = int(params["errors-count-threshold"])
        cluster.kill_problem_processes = bool(int(params["kill-problem-processes"]))
        return cluster

    def to_dict(self):
        return {
            "id": self.id,
            "host": self.host,
            "port": self.port,
            "name": self.name,
            "expiration_timeout": self.expiration_timeout,
            "lifetime_limit": self.lifetime_limit,
            "max_memory_size": self.max_memory_size,
            "max_memory_time_limit": self.max_memory_time_limit,
            "security_level": self.security_level.value,
            "session_fault_tolerance_level": self.session_fault_tolerance_level,
            "load_balancing_mode": self.load_balancing_mode.value,
            "errors_count_threshold": self.errors_count_threshold,
            "kill_problem_processes": self.kill_problem_processes,
        }
